@@ -4,9 +4,9 @@
 
 module Argo.Json.Array where
 
-import qualified Argo.Decoder as Decoder
-import qualified Argo.Encoder as Encoder
 import qualified Argo.Literal as Literal
+import qualified Argo.Type.Decoder as Decoder
+import qualified Argo.Type.Encoder as Encoder
 import qualified Argo.Vendor.Builder as Builder
 import qualified Argo.Vendor.DeepSeq as DeepSeq
 import qualified Argo.Vendor.TemplateHaskell as TH
@@ -24,12 +24,14 @@ toList :: ArrayOf value -> [value]
 toList (Array x) = x
 
 encode :: (value -> Encoder.Encoder ()) -> ArrayOf value -> Encoder.Encoder ()
-encode f = Encoder.list
-    (Trans.lift . Trans.tell $ Builder.word8 Literal.leftSquareBracket)
-    (Trans.lift . Trans.tell $ Builder.word8 Literal.rightSquareBracket)
-    (Trans.lift . Trans.tell $ Builder.word8 Literal.comma)
-    f
-    . toList
+encode f =
+    Encoder.list
+            (Trans.lift . Trans.tell $ Builder.word8 Literal.leftSquareBracket)
+            (Trans.lift . Trans.tell $ Builder.word8 Literal.rightSquareBracket
+            )
+            (Trans.lift . Trans.tell $ Builder.word8 Literal.comma)
+            f
+        . toList
 
 decode :: Decoder.Decoder value -> Decoder.Decoder (ArrayOf value)
 decode f = do
