@@ -11,6 +11,7 @@ import qualified Argo.Json.Number as Number
 import qualified Argo.Json.Object as Object
 import qualified Argo.Json.String as String
 import qualified Argo.Json.Value as Value
+import qualified Argo.Type.Decimal as Decimal
 import qualified Argo.Vendor.Text as Text
 
 pattern Null :: Value.Value
@@ -19,33 +20,27 @@ pattern Null = Value.Null (Null.Null ())
 pattern Boolean :: Bool -> Value.Value
 pattern Boolean x = Value.Boolean (Boolean.Boolean x)
 
-pattern Number :: Integer -> Integer -> Value.Value
-pattern Number x y <- Value.Number (Number.Number x y) where
-    Number x y = Value.Number $ Number.number x y
+pattern Number :: Decimal.Decimal -> Value.Value
+pattern Number x = Value.Number (Number.Number x)
 
 pattern String :: Text.Text -> Value.Value
 pattern String x = Value.String (String.String x)
 
-type Array = [Value.Value]
-
-pattern Array :: Array -> Value.Value
+pattern Array :: [Value.Value] -> Value.Value
 pattern Array x = Value.Array (Array.Array x)
 
-type Object = [Member]
-
-pattern Object :: Object -> Value.Value
+pattern Object :: [Member.MemberOf Value.Value] -> Value.Value
 pattern Object x = Value.Object (Object.Object x)
 
 {-# COMPLETE Null, Boolean, Number, String, Array, Object #-}
 
-type Member = Member.MemberOf Value.Value
-
-pattern Member :: Name.Name -> Value.Value -> Member
-pattern Member k v = Member.Member k v
-
-{-# COMPLETE Member #-}
-
 pattern Name :: Text.Text -> Name.Name
 pattern Name x = Name.Name (String.String x)
 
 {-# COMPLETE Name #-}
+
+pattern Decimal :: Integer -> Integer -> Decimal.Decimal
+pattern Decimal s e <- Decimal.Decimal s e where
+    Decimal s e = Decimal.decimal s e
+
+{-# COMPLETE Decimal #-}
