@@ -14,7 +14,10 @@ module Argo.Vendor.ByteString
     , ByteString.stripPrefix
     , ByteString.take
     , ByteString.takeWhileEnd
+    , LazyByteString.toStrict
     , ByteString.uncons
+    , ByteString.unpack
     ) where
 
 import qualified Data.ByteString as ByteString
+import qualified Data.ByteString.Lazy as LazyByteString
