@@ -1,15 +1,15 @@
-{-# OPTIONS_GHC -Wno-orphans #-}
-
-{-# LANGUAGE FlexibleInstances #-}
 {-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE QuasiQuotes #-}
 
+import Argo.Orphanage ()
 import Data.List.NonEmpty (NonEmpty((:|)))
 import Test.Tasty.HUnit ((@?=))
 import Test.Tasty.QuickCheck ((===))
 
 import qualified Argo
-import qualified Argo.Codec as Codec
+import qualified Argo.Type.Codec as Codec
+import qualified Argo.Type.Permission as Permission
+import qualified Argo.Json.String as String
 import qualified Data.ByteString as ByteString
 import qualified Data.ByteString.Builder as Builder
 import qualified Data.ByteString.Lazy as LazyByteString
@@ -37,19 +37,19 @@ main = Tasty.defaultMain $ Tasty.testGroup "Argo"
             ]
         , Tasty.testGroup "Number"
             [ Tasty.testCase "zero" $ do
-                encode (Argo.Number 0 0) @?= "0"
+                encode (number 0 0) @?= "0"
             , Tasty.testCase "positive integer" $ do
-                encode (Argo.Number 1 0) @?= "1"
+                encode (number 1 0) @?= "1"
             , Tasty.testCase "negative integer" $ do
-                encode (Argo.Number (-1) 0) @?= "-1"
+                encode (number (-1) 0) @?= "-1"
             , Tasty.testCase "positive exponent" $ do
-                encode (Argo.Number 1 1) @?= "1e1"
+                encode (number 1 1) @?= "1e1"
             , Tasty.testCase "negative exponent" $ do
-                encode (Argo.Number 1 (-1)) @?= "1e-1"
+                encode (number 1 (-1)) @?= "1e-1"
             , Tasty.testCase "multiple integer digits" $ do
-                encode (Argo.Number 12 0) @?= "12"
+                encode (number 12 0) @?= "12"
             , Tasty.testCase "multiple exponent digits" $ do
-                encode (Argo.Number 1 12) @?= "1e12"
+                encode (number 1 12) @?= "1e12"
             ]
         , Tasty.testGroup "String"
             [ Tasty.testCase "empty" $ do
@@ -89,20 +89,20 @@ main = Tasty.defaultMain $ Tasty.testGroup "Argo"
             [ Tasty.testCase "empty" $ do
                 encode (Argo.Array []) @?= "[]"
             , Tasty.testCase "one element" $ do
-                encode (Argo.Array [Argo.Number 1 0]) @?= "[1]"
+                encode (Argo.Array [number 1 0]) @?= "[1]"
             , Tasty.testCase "two elements" $ do
-                encode (Argo.Array [Argo.Number 1 0, Argo.Number 2 0]) @?= "[1,2]"
+                encode (Argo.Array [number 1 0, number 2 0]) @?= "[1,2]"
             ]
         , Tasty.testGroup "Object"
             [ Tasty.testCase "empty" $ do
                 encode (Argo.Object []) @?= "{}"
             , Tasty.testCase "one element" $ do
-                encode (Argo.Object [Argo.Member (Argo.Name "a") $ Argo.Number 1 0]) @?= "{\"a\":1}"
+                encode (Argo.Object [Argo.Member (Argo.Name "a") $ number 1 0]) @?= "{\"a\":1}"
             , Tasty.testCase "two elements" $ do
-                encode (Argo.Object [Argo.Member (Argo.Name "a") $ Argo.Number 1 0, Argo.Member (Argo.Name "b") $ Argo.Number 2 0]) @?= "{\"a\":1,\"b\":2}"
+                encode (Argo.Object [Argo.Member (Argo.Name "a") $ number 1 0, Argo.Member (Argo.Name "b") $ number 2 0]) @?= "{\"a\":1,\"b\":2}"
             ]
         ]
-    , Tasty.testGroup "decode" $ let decode = resultToMaybe . Argo.decode :: ByteString.ByteString -> Maybe Argo.Value in
+    , Tasty.testGroup "decode" $ let decode = hush . Argo.decode :: ByteString.ByteString -> Maybe Argo.Value in
         [ Tasty.testGroup "Null"
             [ Tasty.testCase "null" $ do
                 decode "null" @?= Just Argo.Null
@@ -125,37 +125,37 @@ main = Tasty.defaultMain $ Tasty.testGroup "Argo"
             ]
         , Tasty.testGroup "Number"
             [ Tasty.testCase "zero" $ do
-                decode "0" @?= Just (Argo.Number 0 0)
+                decode "0" @?= Just (number 0 0)
             , Tasty.testCase "negative zero" $ do
-                decode "-0" @?= Just (Argo.Number 0 0)
+                decode "-0" @?= Just (number 0 0)
             , Tasty.testCase "positive integer" $ do
-                decode "1" @?= Just (Argo.Number 1 0)
+                decode "1" @?= Just (number 1 0)
             , Tasty.testCase "multiple integer digits" $ do
-                decode "12" @?= Just (Argo.Number 12 0)
+                decode "12" @?= Just (number 12 0)
             , Tasty.testCase "negative integer" $ do
-                decode "-1" @?= Just (Argo.Number (-1) 0)
+                decode "-1" @?= Just (number (-1) 0)
             , Tasty.testCase "fraction" $ do
-                decode "0.1" @?= Just (Argo.Number 1 (-1))
+                decode "0.1" @?= Just (number 1 (-1))
             , Tasty.testCase "multiple fraction digits" $ do
-                decode "0.12" @?= Just (Argo.Number 12 (-2))
+                decode "0.12" @?= Just (number 12 (-2))
             , Tasty.testCase "leading zero fraction" $ do
-                decode "0.01" @?= Just (Argo.Number 1 (-2))
+                decode "0.01" @?= Just (number 1 (-2))
             , Tasty.testCase "positive exponent" $ do
-                decode "1e1" @?= Just (Argo.Number 1 1)
+                decode "1e1" @?= Just (number 1 1)
             , Tasty.testCase "capital exponent" $ do
-                decode "1E1" @?= Just (Argo.Number 1 1)
+                decode "1E1" @?= Just (number 1 1)
             , Tasty.testCase "leading zero exponent" $ do
-                decode "1e01" @?= Just (Argo.Number 1 1)
+                decode "1e01" @?= Just (number 1 1)
             , Tasty.testCase "explicit positive exponent" $ do
-                decode "1e+1" @?= Just (Argo.Number 1 1)
+                decode "1e+1" @?= Just (number 1 1)
             , Tasty.testCase "negative exponent" $ do
-                decode "1e-1" @?= Just (Argo.Number 1 (-1))
+                decode "1e-1" @?= Just (number 1 (-1))
             , Tasty.testCase "multiple exponent digits" $ do
-                decode "1e12" @?= Just (Argo.Number 1 12)
+                decode "1e12" @?= Just (number 1 12)
             , Tasty.testCase "kitchen sink" $ do
-                decode "12.34e56" @?= Just (Argo.Number 1234 54)
+                decode "12.34e56" @?= Just (number 1234 54)
             , Tasty.testCase "normalized" $ do
-                decode "10" @?= Just (Argo.Number 1 1)
+                decode "10" @?= Just (number 1 1)
             , Tasty.testCase "leading zero" $ do
                 decode "01" @?= Nothing
             , Tasty.testCase "trailing fraction" $ do
@@ -241,11 +241,11 @@ main = Tasty.defaultMain $ Tasty.testGroup "Argo"
             [ Tasty.testCase "empty" $ do
                 decode "[]" @?= Just (Argo.Array [])
             , Tasty.testCase "one element" $ do
-                decode "[1]" @?= Just (Argo.Array [Argo.Number 1 0])
+                decode "[1]" @?= Just (Argo.Array [number 1 0])
             , Tasty.testCase "two elements" $ do
-                decode "[1,2]" @?= Just (Argo.Array [Argo.Number 1 0, Argo.Number 2 0])
+                decode "[1,2]" @?= Just (Argo.Array [number 1 0, number 2 0])
             , Tasty.testCase "nested" $ do
-                decode "[1,[2]]" @?= Just (Argo.Array [Argo.Number 1 0, Argo.Array [Argo.Number 2 0]])
+                decode "[1,[2]]" @?= Just (Argo.Array [number 1 0, Argo.Array [number 2 0]])
             , Tasty.testCase "not closed" $ do
                 decode "[" @?= Nothing
             , Tasty.testCase "not opened" $ do
@@ -263,11 +263,11 @@ main = Tasty.defaultMain $ Tasty.testGroup "Argo"
             [ Tasty.testCase "empty" $ do
                 decode "{}" @?= Just (Argo.Object [])
             , Tasty.testCase "one element" $ do
-                decode "{\"a\":1}" @?= Just (Argo.Object [Argo.Member (Argo.Name "a") $ Argo.Number 1 0])
+                decode "{\"a\":1}" @?= Just (Argo.Object [Argo.Member (Argo.Name "a") $ number 1 0])
             , Tasty.testCase "two elements" $ do
-                decode "{\"a\":1,\"b\":2}" @?= Just (Argo.Object [Argo.Member (Argo.Name "a") $ Argo.Number 1 0, Argo.Member (Argo.Name "b") $ Argo.Number 2 0])
+                decode "{\"a\":1,\"b\":2}" @?= Just (Argo.Object [Argo.Member (Argo.Name "a") $ number 1 0, Argo.Member (Argo.Name "b") $ number 2 0])
             , Tasty.testCase "nested" $ do
-                decode "{\"a\":{\"b\":2}}" @?= Just (Argo.Object [Argo.Member (Argo.Name "a") $ Argo.Object [Argo.Member (Argo.Name "b") $ Argo.Number 2 0]])
+                decode "{\"a\":{\"b\":2}}" @?= Just (Argo.Object [Argo.Member (Argo.Name "a") $ Argo.Object [Argo.Member (Argo.Name "b") $ number 2 0]])
             , Tasty.testCase "not closed" $ do
                 decode "{" @?= Nothing
             , Tasty.testCase "not opened" $ do
@@ -292,61 +292,59 @@ main = Tasty.defaultMain $ Tasty.testGroup "Argo"
                 decode "{1:2}" @?= Nothing
             ]
         ]
-    , Tasty.testGroup "fromValue" $
-        let
-            fromValue :: Argo.FromValue a => Argo.Value -> Maybe a
-            fromValue = resultToMaybe . Argo.fromValue
-        in
+    , Tasty.testGroup "fromValue"
         [ Tasty.testCase "Value" $ do
-            fromValue Argo.Null @?= Just Argo.Null
+            Argo.fromValue Argo.Null @?= Right Argo.Null
         , Tasty.testCase "Bool" $ do
-            fromValue (Argo.Boolean False) @?= Just False
+            Argo.fromValue (Argo.Boolean False) @?= Right False
         , Tasty.testCase "Char" $ do
-            fromValue (Argo.String "a") @?= Just 'a'
+            Argo.fromValue (Argo.String "a") @?= Right 'a'
         , Tasty.testCase "Int" $ do
-            fromValue (Argo.Number 0 0) @?= Just (0 :: Int)
+            Argo.fromValue (number 0 0) @?= Right (0 :: Int)
         , Tasty.testCase "Int8" $ do
-            fromValue (Argo.Number 0 0) @?= Just (0 :: Int.Int8)
+            Argo.fromValue (number 0 0) @?= Right (0 :: Int.Int8)
         , Tasty.testCase "Int16" $ do
-            fromValue (Argo.Number 0 0) @?= Just (0 :: Int.Int16)
+            Argo.fromValue (number 0 0) @?= Right (0 :: Int.Int16)
         , Tasty.testCase "Int32" $ do
-            fromValue (Argo.Number 0 0) @?= Just (0 :: Int.Int32)
+            Argo.fromValue (number 0 0) @?= Right (0 :: Int.Int32)
         , Tasty.testCase "Int64" $ do
-            fromValue (Argo.Number 0 0) @?= Just (0 :: Int.Int64)
+            Argo.fromValue (number 0 0) @?= Right (0 :: Int.Int64)
         , Tasty.testCase "Word" $ do
-            fromValue (Argo.Number 0 0) @?= Just (0 :: Word)
+            Argo.fromValue (number 0 0) @?= Right (0 :: Word)
         , Tasty.testCase "Word8" $ do
-            fromValue (Argo.Number 0 0) @?= Just (0 :: Word.Word8)
+            Argo.fromValue (number 0 0) @?= Right (0 :: Word.Word8)
         , Tasty.testCase "Word16" $ do
-            fromValue (Argo.Number 0 0) @?= Just (0 :: Word.Word16)
+            Argo.fromValue (number 0 0) @?= Right (0 :: Word.Word16)
         , Tasty.testCase "Word32" $ do
-            fromValue (Argo.Number 0 0) @?= Just (0 :: Word.Word32)
+            Argo.fromValue (number 0 0) @?= Right (0 :: Word.Word32)
         , Tasty.testCase "Word64" $ do
-            fromValue (Argo.Number 0 0) @?= Just (0 :: Word.Word64)
+            Argo.fromValue (number 0 0) @?= Right (0 :: Word.Word64)
         , Tasty.testCase "Integer" $ do
-            fromValue (Argo.Number 0 0) @?= Just (0 :: Integer)
+            Argo.fromValue (number 0 0) @?= Right (0 :: Integer)
         , Tasty.testCase "Float" $ do
-            fromValue (Argo.Number 0 0) @?= Just (0 :: Float)
+            Argo.fromValue (number 0 0) @?= Right (0 :: Float)
         , Tasty.testCase "Double" $ do
-            fromValue (Argo.Number 0 0) @?= Just (0 :: Double)
+            Argo.fromValue (number 0 0) @?= Right (0 :: Double)
         , Tasty.testCase "String" $ do
-            fromValue (Argo.String "") @?= Just ("" :: String)
+            Argo.fromValue (Argo.String "") @?= Right ("" :: String)
         , Tasty.testCase "Text" $ do
-            fromValue (Argo.String "") @?= Just ("" :: Text.Text)
+            Argo.fromValue (Argo.String "") @?= Right ("" :: Text.Text)
         , Tasty.testCase "LazyText" $ do
-            fromValue (Argo.String "") @?= Just ("" :: LazyText.Text)
+            Argo.fromValue (Argo.String "") @?= Right ("" :: LazyText.Text)
         , Tasty.testCase "Maybe a" $ do
-            fromValue (Argo.Boolean False) @?= Just (Just False)
+            Argo.fromValue (Argo.Boolean False) @?= Right (Just False)
         , Tasty.testCase "()" $ do
-            fromValue (Argo.Array []) @?= Just ()
+            Argo.fromValue (Argo.Array []) @?= Right ()
         , Tasty.testCase "(a, b)" $ do
-            fromValue (Argo.Array [Argo.Boolean False, Argo.String "a"]) @?= Just (False, 'a')
+            Argo.fromValue (Argo.Array [Argo.Boolean False, Argo.String "a"]) @?= Right (False, 'a')
         , Tasty.testCase "[a]" $ do
-            fromValue (Argo.Array []) @?= Just ([] :: [Bool])
+            Argo.fromValue (Argo.Array []) @?= Right ([] :: [Bool])
         , Tasty.testCase "NonEmpty a" $ do
-            fromValue (Argo.Array [Argo.Boolean False]) @?= Just (False :| [])
+            Argo.fromValue (Argo.Array [Argo.Boolean False]) @?= Right (False :| [])
         , Tasty.testCase "Map Text a" $ do
-            fromValue (Argo.Object [Argo.Member (Argo.Name "a") $ Argo.Boolean False]) @?= Just (Map.fromList [("a" :: Text.Text, False)])
+            Argo.fromValue (Argo.Object [Argo.Member (Argo.Name "a") $ Argo.Boolean False]) @?= Right (Map.fromList [("a" :: Text.Text, False)])
+        , Tasty.testCase "Pointer" $ do
+            Argo.fromValue (Argo.String "") @?= Right (Argo.Pointer [])
         ]
     , Tasty.testGroup "toValue"
         [ Tasty.testCase "Value" $ do
@@ -356,31 +354,31 @@ main = Tasty.defaultMain $ Tasty.testGroup "Argo"
         , Tasty.testCase "Char" $ do
             Argo.toValue 'a' @?= Argo.String "a"
         , Tasty.testCase "Int" $ do
-            Argo.toValue (0 :: Int) @?= Argo.Number 0 0
+            Argo.toValue (0 :: Int) @?= number 0 0
         , Tasty.testCase "Int8" $ do
-            Argo.toValue (0 :: Int.Int8) @?= Argo.Number 0 0
+            Argo.toValue (0 :: Int.Int8) @?= number 0 0
         , Tasty.testCase "Int16" $ do
-            Argo.toValue (0 :: Int.Int16) @?= Argo.Number 0 0
+            Argo.toValue (0 :: Int.Int16) @?= number 0 0
         , Tasty.testCase "Int32" $ do
-            Argo.toValue (0 :: Int.Int32) @?= Argo.Number 0 0
+            Argo.toValue (0 :: Int.Int32) @?= number 0 0
         , Tasty.testCase "Int64" $ do
-            Argo.toValue (0 :: Int.Int64) @?= Argo.Number 0 0
+            Argo.toValue (0 :: Int.Int64) @?= number 0 0
         , Tasty.testCase "Word" $ do
-            Argo.toValue (0 :: Word) @?= Argo.Number 0 0
+            Argo.toValue (0 :: Word) @?= number 0 0
         , Tasty.testCase "Word8" $ do
-            Argo.toValue (0 :: Word.Word8) @?= Argo.Number 0 0
+            Argo.toValue (0 :: Word.Word8) @?= number 0 0
         , Tasty.testCase "Word16" $ do
-            Argo.toValue (0 :: Word.Word16) @?= Argo.Number 0 0
+            Argo.toValue (0 :: Word.Word16) @?= number 0 0
         , Tasty.testCase "Word32" $ do
-            Argo.toValue (0 :: Word.Word32) @?= Argo.Number 0 0
+            Argo.toValue (0 :: Word.Word32) @?= number 0 0
         , Tasty.testCase "Word64" $ do
-            Argo.toValue (0 :: Word.Word64) @?= Argo.Number 0 0
+            Argo.toValue (0 :: Word.Word64) @?= number 0 0
         , Tasty.testCase "Integer" $ do
-            Argo.toValue (0 :: Integer) @?= Argo.Number 0 0
+            Argo.toValue (0 :: Integer) @?= number 0 0
         , Tasty.testCase "Float" $ do
-            Argo.toValue (0 :: Float) @?= Argo.Number 0 0
+            Argo.toValue (0 :: Float) @?= number 0 0
         , Tasty.testCase "Double" $ do
-            Argo.toValue (0 :: Double) @?= Argo.Number 0 0
+            Argo.toValue (0 :: Double) @?= number 0 0
         , Tasty.testCase "String" $ do
             Argo.toValue ("" :: String) @?= Argo.String ""
         , Tasty.testCase "Text" $ do
@@ -399,6 +397,8 @@ main = Tasty.defaultMain $ Tasty.testGroup "Argo"
             Argo.toValue (False :| []) @?= Argo.Array [Argo.Boolean False]
         , Tasty.testCase "Map Text a" $ do
             Argo.toValue (Map.fromList [("a" :: Text.Text, False)]) @?= Argo.Object [Argo.Member (Argo.Name "a") $ Argo.Boolean False]
+        , Tasty.testCase "Pointer" $ do
+            Argo.toValue (Argo.Pointer []) @?= Argo.String ""
         ]
     , Tasty.testGroup "quasi quoter"
         [ Tasty.testCase "Null" $ do
@@ -406,7 +406,7 @@ main = Tasty.defaultMain $ Tasty.testGroup "Argo"
         , Tasty.testCase "Boolean" $ do
             [Argo.value| false |] @?= Argo.Boolean False
         , Tasty.testCase "Number" $ do
-            [Argo.value| 0 |] @?= Argo.Number 0 0
+            [Argo.value| 0 |] @?= number 0 0
         , Tasty.testCase "String" $ do
             [Argo.value| "" |] @?= Argo.String ""
         , Tasty.testCase "Array" $ do
@@ -416,109 +416,170 @@ main = Tasty.defaultMain $ Tasty.testGroup "Argo"
         ]
     , Tasty.testGroup "property"
         [ property "decode . encode" $ \ x ->
-            (resultToMaybe . Argo.decode . LazyByteString.toStrict . Builder.toLazyByteString $ Argo.encode x) === Just (x :: Argo.Value)
+            (Argo.decode . LazyByteString.toStrict . Builder.toLazyByteString $ Argo.encode x) === Right (x :: Argo.Value)
         , property "decode . encodeWith" $ \ x ->
-            (Argo.decode . LazyByteString.toStrict . Builder.toLazyByteString $ Argo.encodeWith Argo.Tab x) === Argo.Success (x :: Argo.Value)
+            (Argo.decode . LazyByteString.toStrict . Builder.toLazyByteString $ Argo.encodeWith Argo.Tab x) === Right (x :: Argo.Value)
         , Tasty.testGroup "fromValue . toValue"
             [ property "Value" $ \ x ->
-                (resultToMaybe . Argo.fromValue $ Argo.toValue x) === Just (x :: Argo.Value)
+                Argo.fromValue (Argo.toValue x) === Right (x :: Argo.Value)
             , property "Bool" $ \ x ->
-                Argo.fromValue (Argo.toValue x) === Argo.Success (x :: Bool)
+                Argo.fromValue (Argo.toValue x) === Right (x :: Bool)
             , property "Char" $ \ x ->
                 Argo.fromValue (Argo.toValue x) === if '\xd800' <= x && x <= '\xdfff'
-                    then Argo.Success '\xfffd'
-                    else Argo.Success x
+                    then Right '\xfffd'
+                    else Right x
             , property "Int" $ \ x ->
-                Argo.fromValue (Argo.toValue x) === Argo.Success (x :: Int)
+                Argo.fromValue (Argo.toValue x) === Right (x :: Int)
             , property "Int8" $ \ x ->
-                Argo.fromValue (Argo.toValue x) === Argo.Success (x :: Int.Int8)
+                Argo.fromValue (Argo.toValue x) === Right (x :: Int.Int8)
             , property "Int16" $ \ x ->
-                Argo.fromValue (Argo.toValue x) === Argo.Success (x :: Int.Int16)
+                Argo.fromValue (Argo.toValue x) === Right (x :: Int.Int16)
             , property "Int32" $ \ x ->
-                Argo.fromValue (Argo.toValue x) === Argo.Success (x :: Int.Int32)
+                Argo.fromValue (Argo.toValue x) === Right (x :: Int.Int32)
             , property "Int64" $ \ x ->
-                Argo.fromValue (Argo.toValue x) === Argo.Success (x :: Int.Int64)
+                Argo.fromValue (Argo.toValue x) === Right (x :: Int.Int64)
             , property "Word" $ \ x ->
-                Argo.fromValue (Argo.toValue x) === Argo.Success (x :: Word)
+                Argo.fromValue (Argo.toValue x) === Right (x :: Word)
             , property "Word8" $ \ x ->
-                Argo.fromValue (Argo.toValue x) === Argo.Success (x :: Word.Word8)
+                Argo.fromValue (Argo.toValue x) === Right (x :: Word.Word8)
             , property "Word16" $ \ x ->
-                Argo.fromValue (Argo.toValue x) === Argo.Success (x :: Word.Word16)
+                Argo.fromValue (Argo.toValue x) === Right (x :: Word.Word16)
             , property "Word32" $ \ x ->
-                Argo.fromValue (Argo.toValue x) === Argo.Success (x :: Word.Word32)
+                Argo.fromValue (Argo.toValue x) === Right (x :: Word.Word32)
             , property "Word64" $ \ x ->
-                Argo.fromValue (Argo.toValue x) === Argo.Success (x :: Word.Word64)
+                Argo.fromValue (Argo.toValue x) === Right (x :: Word.Word64)
             , property "Integer" $ \ x ->
-                Argo.fromValue (Argo.toValue x) === Argo.Success (x :: Integer)
+                Argo.fromValue (Argo.toValue x) === Right (x :: Integer)
             , property "Float" $ \ x ->
                 Argo.fromValue (Argo.toValue x) === if isNaN x || isInfinite x
-                    then Argo.Failure "expected Float but got Null (Null ())"
-                    else Argo.Success (x :: Float)
+                    then Left "expected Float but got Null (Null ())"
+                    else Right (x :: Float)
             , property "Double" $ \ x ->
                 Argo.fromValue (Argo.toValue x) === if isNaN x || isInfinite x
-                    then Argo.Failure "expected Double but got Null (Null ())"
-                    else Argo.Success (x :: Double)
+                    then Left "expected Double but got Null (Null ())"
+                    else Right (x :: Double)
             , property "String" $ \ x ->
-                Argo.fromValue (Argo.toValue x) === Argo.Success (Text.unpack $ Text.pack x)
+                Argo.fromValue (Argo.toValue x) === Right (Text.unpack $ Text.pack x)
             , property "Text" $ \ x ->
-                Argo.fromValue (Argo.toValue x) === Argo.Success (x :: Text.Text)
+                Argo.fromValue (Argo.toValue x) === Right (x :: Text.Text)
             , property "LazyText" $ \ x ->
-                Argo.fromValue (Argo.toValue x) === Argo.Success (x :: LazyText.Text)
+                Argo.fromValue (Argo.toValue x) === Right (x :: LazyText.Text)
             , property "Maybe a" $ \ x ->
-                Argo.fromValue (Argo.toValue x) === Argo.Success (x :: Maybe Bool)
+                Argo.fromValue (Argo.toValue x) === Right (x :: Maybe Bool)
             , property "()" $ \ x ->
-                Argo.fromValue (Argo.toValue x) === Argo.Success (x :: ())
+                Argo.fromValue (Argo.toValue x) === Right (x :: ())
             , property "(a, b)" $ \ x ->
-                Argo.fromValue (Argo.toValue x) === Argo.Success (x :: (Bool, Int.Int8))
+                Argo.fromValue (Argo.toValue x) === Right (x :: (Bool, Int.Int8))
             , property "[a]" $ \ x ->
-                Argo.fromValue (Argo.toValue x) === Argo.Success (x :: [Bool])
+                Argo.fromValue (Argo.toValue x) === Right (x :: [Bool])
             , property "NonEmpty a" $ \ x ->
-                Argo.fromValue (Argo.toValue x) === Argo.Success (x :: NonEmpty Bool)
+                Argo.fromValue (Argo.toValue x) === Right (x :: NonEmpty Bool)
             , property "Map Text a" $ \ x ->
-                Argo.fromValue (Argo.toValue x) === Argo.Success (x :: Map.Map Text.Text Bool)
+                Argo.fromValue (Argo.toValue x) === Right (x :: Map.Map Text.Text Bool)
+            , property "Pointer" $ \ x ->
+                Argo.fromValue (Argo.toValue x) === Right (x :: Argo.Pointer)
             ]
         ]
     , Tasty.testGroup "Codec"
         [ Tasty.testCase "encode text" $ do
             Codec.encodeWith Codec.textCodec "" @?= Argo.String ""
         , Tasty.testCase "decode text" $ do
-            Codec.decodeWith Codec.textCodec (Argo.String "") @?= Argo.Success ""
+            Codec.decodeWith Codec.textCodec (Argo.String "") @?= Right ""
         , Tasty.testCase "encode bool" $ do
             Codec.encodeWith Codec.boolCodec False @?= Argo.Boolean False
         , Tasty.testCase "decode bool" $ do
-            Codec.decodeWith Codec.boolCodec (Argo.Boolean False) @?= Argo.Success False
+            Codec.decodeWith Codec.boolCodec (Argo.Boolean False) @?= Right False
         , Tasty.testCase "encode maybe text" $ do
             Codec.encodeWith (Codec.maybeCodec Codec.textCodec) Nothing @?= Argo.Null
             Codec.encodeWith (Codec.maybeCodec Codec.textCodec) (Just "") @?= Argo.String ""
         , Tasty.testCase "decode maybe text" $ do
-            Codec.decodeWith (Codec.maybeCodec Codec.textCodec) Argo.Null @?= Argo.Success Nothing
-            Codec.decodeWith (Codec.maybeCodec Codec.textCodec) (Argo.String "") @?= Argo.Success (Just "")
+            Codec.decodeWith (Codec.maybeCodec Codec.textCodec) Argo.Null @?= Right Nothing
+            Codec.decodeWith (Codec.maybeCodec Codec.textCodec) (Argo.String "") @?= Right (Just "")
         , Tasty.testCase "encode either text bool" $ do
             Codec.encodeWith (Codec.eitherCodec Codec.textCodec Codec.boolCodec) (Left "") @?= Argo.Object [Argo.Member (Argo.Name "type") $ Argo.String "Left", Argo.Member (Argo.Name "value") $ Argo.String ""]
             Codec.encodeWith (Codec.eitherCodec Codec.textCodec Codec.boolCodec) (Right False) @?= Argo.Object [Argo.Member (Argo.Name "type") $ Argo.String "Right", Argo.Member (Argo.Name "value") $ Argo.Boolean False]
         , Tasty.testCase "decode either text bool" $ do
-            Codec.decodeWith (Codec.eitherCodec Codec.textCodec Codec.boolCodec) (Argo.Object [Argo.Member (Argo.Name "type") $ Argo.String "Left", Argo.Member (Argo.Name "value") $ Argo.String ""]) @?= Argo.Success (Left "")
-            Codec.decodeWith (Codec.eitherCodec Codec.textCodec Codec.boolCodec) (Argo.Object [Argo.Member (Argo.Name "type") $ Argo.String "Right", Argo.Member (Argo.Name "value") $ Argo.Boolean False]) @?= Argo.Success (Right False)
+            Codec.decodeWith (Codec.eitherCodec Codec.textCodec Codec.boolCodec) (Argo.Object [Argo.Member (Argo.Name "type") $ Argo.String "Left", Argo.Member (Argo.Name "value") $ Argo.String ""]) @?= Right (Left "")
+            Codec.decodeWith (Codec.eitherCodec Codec.textCodec Codec.boolCodec) (Argo.Object [Argo.Member (Argo.Name "type") $ Argo.String "Right", Argo.Member (Argo.Name "value") $ Argo.Boolean False]) @?= Right (Right False)
         , Tasty.testCase "encode tuple text bool" $ do
             Codec.encodeWith (Codec.tupleCodec Codec.textCodec Codec.boolCodec) ("", False) @?= Argo.Array [Argo.String "", Argo.Boolean False]
         , Tasty.testCase "decode tuple text bool" $ do
-            Codec.decodeWith (Codec.tupleCodec Codec.textCodec Codec.boolCodec) (Argo.Array [Argo.String "", Argo.Boolean False]) @?= Argo.Success ("", False)
+            Codec.decodeWith (Codec.tupleCodec Codec.textCodec Codec.boolCodec) (Argo.Array [Argo.String "", Argo.Boolean False]) @?= Right ("", False)
         , Tasty.testCase "encode record" $ do
             Codec.encodeWith recordCodec (Record False Nothing) @?= Argo.Object [Argo.Member (Argo.Name "bool") $ Argo.Boolean False]
             Codec.encodeWith recordCodec (Record False $ Just "") @?= Argo.Object [Argo.Member (Argo.Name "bool") $ Argo.Boolean False, Argo.Member (Argo.Name "text") $ Argo.String ""]
         , Tasty.testCase "decode record" $ do
-            Codec.decodeWith recordCodec (Argo.Object [Argo.Member (Argo.Name "bool") $ Argo.Boolean False]) @?= Argo.Success (Record False Nothing)
-            Codec.decodeWith recordCodec (Argo.Object [Argo.Member (Argo.Name "bool") $ Argo.Boolean False, Argo.Member (Argo.Name "text") $ Argo.String ""]) @?= Argo.Success (Record False $ Just "")
+            Codec.decodeWith recordCodec (Argo.Object [Argo.Member (Argo.Name "bool") $ Argo.Boolean False]) @?= Right (Record False Nothing)
+            Codec.decodeWith recordCodec (Argo.Object [Argo.Member (Argo.Name "bool") $ Argo.Boolean False, Argo.Member (Argo.Name "text") $ Argo.String ""]) @?= Right (Record False $ Just "")
+        ]
+    , Tasty.testGroup "Pointer"
+        $ let pointer = Argo.Pointer . fmap Argo.Token in
+        [ Tasty.testCase "decode" $ do
+            let decode = hush . Argo.decodePointer
+            decode "" @?= Just (pointer [])
+            decode "/" @?= Just (pointer [""])
+            decode "/a" @?= Just (pointer ["a"])
+            decode "/a/b" @?= Just (pointer ["a", "b"])
+            decode "/ab" @?= Just (pointer ["ab"])
+            decode "/~0" @?= Just (pointer ["~"])
+            decode "/~1" @?= Just (pointer ["/"])
+            decode "/~01" @?= Just (pointer ["~1"])
+            decode "a" @?= Nothing
+            decode "/~2" @?= Nothing
+        , Tasty.testCase "encode" $ do
+            let encode = Builder.toLazyByteString . Argo.encodePointer
+            encode (pointer []) @?= ""
+            encode (pointer [""]) @?= "/"
+            encode (pointer ["a"]) @?= "/a"
+            encode (pointer ["a", "b"]) @?= "/a/b"
+            encode (pointer ["ab"]) @?= "/ab"
+            encode (pointer ["~"]) @?= "/~0"
+            encode (pointer ["/"]) @?= "/~1"
+            encode (pointer ["~1"]) @?= "/~01"
+        , property "decode . encode" $ \ x ->
+            (Argo.decodePointer . LazyByteString.toStrict . Builder.toLazyByteString $ Argo.encodePointer x) === Right (x :: Argo.Pointer)
+        , Tasty.testCase "quasi-quoter" $ do
+            [Argo.pointer||] @?= pointer []
+            [Argo.pointer|/|] @?= pointer [""]
+            [Argo.pointer|/a|] @?= pointer ["a"]
+            [Argo.pointer|/a/b|] @?= pointer ["a", "b"]
+        , Tasty.testCase "evaluate" $ do
+            let evaluate p = hush . Argo.evaluate p
+            evaluate (pointer []) Argo.Null @?= Just Argo.Null
+            evaluate (pointer ["a"]) (Argo.Object [Argo.Member (Argo.Name "a") Argo.Null]) @?= Just Argo.Null
+            evaluate (pointer ["a", "b"]) (Argo.Object [Argo.Member (Argo.Name "a") $ Argo.Object [Argo.Member (Argo.Name "b") Argo.Null]]) @?= Just Argo.Null
+            evaluate (pointer ["0"]) (Argo.Object [Argo.Member (Argo.Name "0") Argo.Null]) @?= Just Argo.Null
+            evaluate (pointer ["0"]) (Argo.Array [Argo.Null]) @?= Just Argo.Null
+            evaluate (pointer ["0", "1"]) (Argo.Array [Argo.Array [Argo.Boolean False, Argo.Null]]) @?= Just Argo.Null
+            evaluate (pointer ["a"]) Argo.Null @?= Nothing
+            evaluate (pointer ["a"]) (Argo.Object []) @?= Nothing
+            evaluate (pointer ["a"]) (Argo.Array [Argo.Null]) @?= Nothing
+            evaluate (pointer ["0"]) (Argo.Array []) @?= Nothing
+            evaluate (pointer ["-"]) (Argo.Array []) @?= Nothing
+            evaluate (pointer ["00"]) (Argo.Array [Argo.Null]) @?= Nothing
+        ]
+    , Tasty.testGroup "OverloadedStrings"
+        [ Tasty.testCase "Value" $ do
+            "" @?= Argo.String ""
+        , Tasty.testCase "String" $ do
+            "" @?= String.String ""
+        , Tasty.testCase "Name" $ do
+            "" @?= Argo.Name ""
+        , Tasty.testCase "Token" $ do
+            "" @?= Argo.Token ""
         ]
     ]
 
+number :: Integer -> Integer -> Argo.Value
+number s = Argo.Number . Argo.Decimal s
+
 data Record = Record
     { recordBool :: Bool
     , recordText :: Maybe Text.Text
     } deriving (Eq, Show)
 
 recordCodec :: Codec.ValueCodec Record
-recordCodec = Codec.fromObjectCodec Codec.Allow $ Record
+recordCodec = Codec.fromObjectCodec Permission.Allow $ Record
     <$> Codec.project recordBool (Codec.required (Argo.Name "bool") Codec.boolCodec)
     <*> Codec.project recordText (Codec.optional (Argo.Name "text") Codec.textCodec)
 
@@ -540,47 +601,5 @@ propertyWith n g s f = Tasty.testProperty n
     . Tasty.forAll g
     $ \ x -> Tasty.shrinking s x f
 
-instance Tasty.Arbitrary Argo.Name where
-    arbitrary = Argo.Name <$> Tasty.arbitrary
-    shrink (Argo.Name x) = Argo.Name <$> Tasty.shrink x
-
-instance Tasty.Arbitrary Argo.Member where
-    arbitrary = Argo.Member <$> Tasty.arbitrary <*> Tasty.arbitrary
-    shrink (Argo.Member k v) = uncurry Argo.Member <$> Tasty.shrink (k, v)
-
-instance Tasty.Arbitrary Argo.Value where
-    arbitrary = Tasty.sized genValueSized
-    shrink x = case x of
-        Argo.Null -> []
-        Argo.Boolean y -> Argo.Boolean <$> Tasty.shrink y
-        Argo.Number y z -> uncurry Argo.Number <$> Tasty.shrink (y, z)
-        Argo.String y -> Argo.String <$> Tasty.shrink y
-        Argo.Array y -> Argo.Array <$> Tasty.shrink y
-        Argo.Object y -> Argo.Object <$> Tasty.shrink y
-
-genValueSized :: Int -> Tasty.Gen Argo.Value
-genValueSized size = let newSize = div size 3 in Tasty.oneof
-    [ pure Argo.Null
-    , Argo.Boolean <$> Tasty.arbitrary
-    , Argo.Number <$> Tasty.arbitrary <*> Tasty.arbitrary
-    , Argo.String <$> Tasty.arbitrary
-    , Argo.Array <$> Tasty.vectorOf size (genValueSized newSize)
-    , Argo.Object <$> Tasty.vectorOf size (Argo.Member <$> Tasty.arbitrary <*> genValueSized newSize)
-    ]
-
-resultToMaybe :: Argo.Result a -> Maybe a
-resultToMaybe r = case r of
-    Argo.Failure _ -> Nothing
-    Argo.Success x -> Just x
-
-instance Tasty.Arbitrary Text.Text where
-    arbitrary = Text.pack <$> Tasty.arbitrary
-    shrink = Tasty.shrinkMap Text.pack Text.unpack
-
-instance Tasty.Arbitrary LazyText.Text where
-    arbitrary = LazyText.pack <$> Tasty.arbitrary
-    shrink = Tasty.shrinkMap LazyText.pack LazyText.unpack
-
-instance Tasty.Arbitrary a => Tasty.Arbitrary (NonEmpty a) where
-    arbitrary = (:|) <$> Tasty.arbitrary <*> Tasty.arbitrary
-    shrink (x :| xs) = uncurry (:|) <$> Tasty.shrink (x, xs)
+hush :: Either String a -> Maybe a
+hush = either (const Nothing) Just
